@@ -128,6 +128,19 @@ class Test(unittest.TestCase):
         self.assertGreaterEqual(d.multi_gpu_board_group_id, 0)
         self.assertGreaterEqual(d.max_pitch, 0)
 
+    def test_extract_ptr(self):
+        a = numpy.zeros(127, dtype=numpy.float32)
+        ptr = cu.CU.extract_ptr(a)
+        self.assertEqual(ptr, int(a.__array_interface__["data"][0]))
+        ptr2, sz = cu.CU.extract_ptr_and_size(a, None)
+        self.assertEqual(ptr, ptr2)
+        self.assertEqual(sz, a.nbytes)
+        ptr = cu.CU.extract_ptr(None)
+        self.assertEqual(ptr, 0)
+        ptr2, sz = cu.CU.extract_ptr_and_size(None, 0)
+        self.assertEqual(ptr2, 0)
+        self.assertEqual(sz, 0)
+
     def test_dump_devices(self):
         logging.debug("ENTER: test_dump_devices")
         logging.debug("Available CUDA devices:\n%s",
@@ -173,12 +186,12 @@ class Test(unittest.TestCase):
 
         h, h0 = self._run_on_thread(self._check_push_pop, (ctx,))
         self.assertEqual(h, ctx.handle)
-        self.assertEqual(h0, cu.NULL)
+        self.assertEqual(h0, 0)
         logging.debug("push/pop succeeded")
 
         h, h0 = self._run_on_thread(self._check_with, (ctx,))
         self.assertEqual(h, ctx.handle)
-        self.assertEqual(h0, cu.NULL)
+        self.assertEqual(h0, 0)
         logging.debug("with succeeded")
 
         self.assertEqual(
@@ -420,8 +433,8 @@ class Test(unittest.TestCase):
     def test_memcpy_3d_async(self):
         logging.debug("ENTER: test_memcpy_3d_async")
 
-        p_copy = cu.ffi.new("CUDA_MEMCPY3D *")
-        self.assertEqual(cu.ffi.sizeof(p_copy[0]), 200)
+        p_copy = cu.get_ffi().new("CUDA_MEMCPY3D *")
+        self.assertEqual(cu.get_ffi().sizeof(p_copy[0]), 200)
 
         ctx = cu.Devices().create_some_context()
         logging.debug("Context created")
